var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { makeKMap, AsyncModels, } from '..';
import { sleep } from '../helpers/sleep.func';
export class CloudSynchronizer extends AsyncModels.keyValue {
    constructor(cloudOptions) {
        super({
            name: '_cloudSynchronizer',
            driver: cloudOptions.driver,
        });
        this.models = makeKMap();
        this.cloudOptions = cloudOptions;
        for (const model of cloudOptions.models) {
            this.models[model.options.name] = model;
            model.setItem$((value, key) => __awaiter(this, void 0, void 0, function* () {
                yield this._pushWaitingPushData([
                    {
                        type: 'put',
                        modelName: model.options.name,
                        timestampMs: new Date().getTime(),
                        key: key,
                        value: value,
                        pushing: false,
                    },
                ]);
            }));
            model.removeItem$((key) => __awaiter(this, void 0, void 0, function* () {
                yield this._pushWaitingPushData([
                    {
                        type: 'forget',
                        modelName: model.options.name,
                        timestampMs: new Date().getTime(),
                        key: key,
                        value: null,
                        pushing: false,
                    },
                ]);
                if (true === this.cloudOptions.autoPush) {
                    yield this.push();
                }
            }));
            if ('insertItem' in model) {
                model.insertItem$((value, key) => __awaiter(this, void 0, void 0, function* () {
                    yield this._pushWaitingPushData([
                        {
                            type: 'put',
                            modelName: model.options.name,
                            timestampMs: new Date().getTime(),
                            key: key,
                            value: value,
                            pushing: false,
                        },
                    ]);
                    if (true === this.cloudOptions.autoPush) {
                        yield this.push();
                    }
                }));
            }
        }
    }
    push(_waitingPushData, maxRetries) {
        return __awaiter(this, void 0, void 0, function* () {
            if (undefined === maxRetries) {
                maxRetries = this.cloudOptions.maxRetries;
            }
            if (undefined === _waitingPushData) {
                _waitingPushData = yield this._processWaitingPushData();
            }
            if (false === (yield this.cloudOptions.pushDataHandler(_waitingPushData))) {
                yield sleep(this.cloudOptions.retryDelayInterval);
                --maxRetries;
                if (0 >= maxRetries) {
                    // maxRetries is exhausted.
                    yield this._processedWaitingPushData(_waitingPushData, true);
                    return;
                }
                return yield this.push(_waitingPushData, maxRetries);
            }
            else {
                yield this._processedWaitingPushData(_waitingPushData, true);
                return;
            }
        });
    }
    _pushWaitingPushData(item) {
        return __awaiter(this, void 0, void 0, function* () {
            let _waitingPushData = yield this.getItem('_waitingPushData');
            if (undefined === _waitingPushData || null === _waitingPushData) {
                _waitingPushData = [];
            }
            _waitingPushData.push(...item);
            yield this.setItem('_waitingPushData', _waitingPushData);
        });
    }
    _processWaitingPushData() {
        return __awaiter(this, void 0, void 0, function* () {
            let _waitingPushData = yield this.getItem('_waitingPushData');
            if (undefined === _waitingPushData || null === _waitingPushData) {
                _waitingPushData = [];
            }
            let newWaitingPushData = [];
            _waitingPushData.forEach((item) => {
                if (true === item.pushing)
                    return;
                item.pushing = true;
                newWaitingPushData.push(Object.assign({}, item));
            });
            yield this.setItem('_waitingPushData', _waitingPushData);
            return newWaitingPushData;
        });
    }
    _processedWaitingPushData(waitingPushData, isSuccess) {
        return __awaiter(this, void 0, void 0, function* () {
            let _waitingPushData = yield this.getItem('_waitingPushData');
            if (undefined === _waitingPushData || null === _waitingPushData) {
                _waitingPushData = [];
            }
            for (const i in _waitingPushData) {
                for (const j in waitingPushData) {
                    if (_waitingPushData[i].timestampMs !== waitingPushData[j].timestampMs)
                        continue;
                    if (_waitingPushData[i].modelName !== waitingPushData[j].modelName)
                        continue;
                    if (_waitingPushData[i].type !== waitingPushData[j].type)
                        continue;
                    if (_waitingPushData[i].key !== waitingPushData[j].key)
                        continue;
                    if (true !== _waitingPushData[i].pushing)
                        continue;
                    if (true === isSuccess)
                        delete _waitingPushData[i];
                    else
                        _waitingPushData[i].pushing = false;
                }
            }
            yield this.setItem('_waitingPushData', _waitingPushData);
        });
    }
}
// export class CloudSynchronizer {
//   options: CloudSynchronizerOptionsInterface;
//   models = makeKMap<
//     | SyncCollectionModel
//     | SyncKeyValueModel
//     | AsyncCollectionModel
//     | AsyncKeyValueModel
//   >();
//   private _waitingPushData: PushData = [];
//   constructor(options: CloudSynchronizerOptionsInterface) {
//     this.options = options;
//     for (const model of options.models) {
//       this.models[model.options.name] = model;
//       model.setItem$((value, key) => {
//         this._pushWaitingPushData({
//           type: 'put',
//           modelName: model.options.name,
//           timestampMs: new Date().getTime(),
//           key: key,
//           value: value,
//         });
//       });
//       model.removeItem$((key) => {
//         this._pushWaitingPushData({
//           type: 'forget',
//           modelName: model.options.name,
//           timestampMs: new Date().getTime(),
//           key: key,
//           value: null,
//         });
//       });
//       if ('insertItem' in model) {
//         model.insertItem$((value, key) => {
//           this._pushWaitingPushData({
//             type: 'put',
//             modelName: model.options.name,
//             timestampMs: new Date().getTime(),
//             key: key,
//             value: value,
//           });
//         });
//       }
//     }
//   }
//   private _pushWaitingPushData(item: PushDataItem) {
//     this._pushWaitingPushData(item);
//     // this.options.driver.make();
//   }
//   async push(_waitingPushData?: PushData, maxRetries?: number): Promise<void> {
//     if (undefined === maxRetries) {
//       maxRetries = this.options.maxRetries;
//     }
//     if (undefined === _waitingPushData) {
//       _waitingPushData = this._waitingPushData;
//       this._waitingPushData = [];
//     }
//     if (false === (await this.options.pushDataHandler(_waitingPushData))) {
//       await sleep(this.options.retryDelayInterval);
//       --maxRetries;
//       if (0 >= maxRetries) {
//         // maxRetries is exhausted.
//         this._pushWaitingPushData(..._waitingPushData);
//         return;
//       }
//       return await this.push(_waitingPushData, maxRetries);
//     }
//   }
//   async pull() {}
// }
//# sourceMappingURL=cloud-synchronizer.class.js.map