export declare function makeKMap<ValueType = any>(map?: Map<string, unknown>): KMap<ValueType>;
/**
 * KMap is a collection of like objects, the difference is that it will be traversed in strict insertion order.
 */
export declare type KMap<T = any> = {
    toArray: () => Array<T>;
    toMap: () => Map<string, T>;
    getLength: () => number;
    getKeys: () => IterableIterator<string>;
    getValues: () => IterableIterator<T>;
    forEach: (cfn: (value: T, key: string) => void) => void;
    sortBy: (compareFn: (a: any, b: any) => number) => KMap<T>;
    paginate: (currentPpage: number, num: number, compareFn?: (a: any, b: any) => number) => KMap<T>;
    /**
     * Like Array "at" function, return key.
     */
    getAtKey: (index: number) => string | undefined;
    /**
     * Like Array "at" function, return value.
     */
    getAtValue: (index: number) => T | undefined;
} & {
    [Symbol.iterator](): IterableIterator<T>;
    [other: string]: T;
};
