var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { globalConfig } from '../../providers';
import { BaseModel } from './base-model.class';
export class CollectionModel extends BaseModel {
    constructor(options) {
        super(Object.assign(Object.assign({ autoIncrementHandler: globalConfig.asyncAutoIncrementHandler }, options), { ioType: 'async', modelType: 'collection' }));
        this.nextPrimaryKey = 1;
        this.seeded = false;
        this._curdSubscribeFuncs = {
            insertItem: {},
            setItem: {},
            removeItem: {},
        };
    }
    insertItem(item) {
        return __awaiter(this, void 0, void 0, function* () {
            let skey;
            if (undefined === this.storage) {
                if (undefined === this.options.autoIncrementHandler)
                    skey = String(this.nextPrimaryKey++);
                else
                    skey = yield this.options.autoIncrementHandler(this.options);
                this.cache.publish(skey, item);
            }
            else {
                skey = yield this.storage.insertAutoIncrement(item);
            }
            // trigger subscriptions
            for (const id in this._curdSubscribeFuncs.insertItem) {
                const closFunc = this._curdSubscribeFuncs.insertItem[id];
                closFunc(item, skey);
            }
            return skey;
        });
    }
    bulkInsertItem(items) {
        return __awaiter(this, void 0, void 0, function* () {
            if (undefined === this.storage) {
                const keys = [];
                for (const key in items) {
                    const skey = String(this.nextPrimaryKey++);
                    this.cache.publish(skey, items[key]);
                    keys.push(skey);
                    // trigger subscriptions
                    for (const id in this._curdSubscribeFuncs.insertItem) {
                        const closFunc = this._curdSubscribeFuncs.insertItem[id];
                        closFunc(items[key], key);
                    }
                }
                return keys;
            }
            else {
                const keys = yield this.storage.bulkInsertAutoIncrement(items);
                // trigger subscriptions
                for (const id in this._curdSubscribeFuncs.insertItem) {
                    const closFunc = this._curdSubscribeFuncs.insertItem[id];
                    for (const i in keys) {
                        closFunc(items[i], keys[i]);
                    }
                }
                return keys;
            }
        });
    }
    insertItem$(closFunc) {
        const id = ++this._curdSubscribeId;
        this._curdSubscribeFuncs.insertItem[id] = closFunc;
        return () => delete this._curdSubscribeFuncs.insertItem[id];
    }
    all() {
        if (undefined !== this.storage) {
            throw new Error(`[Kurimudb] This model uses a storage driver, try using the "all" function provided by the driver.`);
        }
        return this.cache.all();
    }
    seed(seed) {
        return __awaiter(this, void 0, void 0, function* () {
            let seedFunc;
            if ('function' === typeof seed) {
                seedFunc = () => {
                    this.seeded = true;
                    seed();
                };
            }
            else if (seed instanceof Array) {
                seedFunc = () => {
                    this.seeded = true;
                    this.bulkInsertItem(seed);
                };
            }
            else {
                throw new Error(`[Kurimudb] In "keyValue" model, the argument to the seed function must be "Function" or "Array".`);
            }
            if (this.seeded)
                return;
            if (undefined === this.storage) {
                yield seedFunc();
                return;
            }
            const storage = this.storage;
            yield storage.seeding(seedFunc);
        });
    }
}
//# sourceMappingURL=collection-model.class.js.map